/*NB_REVISION*/

/*NB_COPYRIGHT*/

#include <stdio.h>
#include <stdlib.h>
#include <constants.h>

#include "effs_std.h"
#include "file/flashdrv.h"
#include "fs_main.h"

int fs_phy_OnChipFlash(FS_FLASH *flash);

/**
 *  fs_main
 *
 *  Initializes the flash file system.
 */
void fs_main(void)
{
    iprintf("File system version:  %s\r\n", fs_getversion());
    fs_init();   // Initialize the file system

    {
        long mem_size = 0;
        char *mem_ptr = nullptr;
        int rc = 0;

        iprintf("Initializing file system...");
        // Returns the number of bytes of ram memory required for the file system.
        mem_size = fs_getmem_flashdrive(fs_phy_OnChipFlash);
        iprintf(" %s (%ld bytes)\r\n", (mem_size > 0) ? "Success" : "Failed", mem_size);
        if (mem_size)
        {
            iprintf("Allocating memory...");
            mem_ptr = (char *)malloc(mem_size);
            iprintf(" %s\r\n", (mem_ptr != NULL) ? "Success" : "Failed");
            if (mem_ptr)
            {
                // Mount the file system.
                iprintf("Mounting drive...");
                rc = fd_mountstd(NOR_DRV_NUM, mem_ptr, mem_size, fs_mount_flashdrive, fs_phy_OnChipFlash);
                iprintf(" %s\r\n", (rc == FS_NO_ERROR) ? "Success" : "Failed");

                if (rc)   // If mount was not successful, then format the drive
                {
                    iprintf("Mount failed with error code: %d, formatting drive...", rc);
                    rc = fd_format(NOR_DRV_NUM, 0);
                    iprintf(" %s\r\n", (rc == FS_NO_ERROR) ? "Success" : "Failed");
                }
            }
        }
    }

    fd_chdrive(NOR_DRV_NUM);
}
